<?php
/**
 * VTBPay Module Integration
 *
 * This script defines a hook function for integrating the VTBPay payment system with the PHPShop module.
 * It handles payment processing, API requests, and logging for the VTBPay integration.
 *
 * @package VtbPay
 */
//   autoload,     payment/result.php
include_once($GLOBALS['SysValue']['class']['composer']);

use \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException;


/**
 * VTBPay Payment Hook
 *
 * This function is a hook for processing payments through the VTBPay payment system.
 *
 * @param object $obj The PHPShop order object for the current order.
 * @param array $value The array containing order-related values.
 * @param string $rout The routing information for the current order.
 *
 * @return void This function doesn't return a value directly, but it modifies the $obj parameter with payment information.
 *
 * @throws \Exception If there are errors during payment processing or API communication.
 */
function send_to_order_mod_vtbpay_hook($obj, $value, $rout)
{
    // Initialize PHPShop ORM and get settings
    $PHPShopOrm = new PHPShopOrm($GLOBALS['SysValue']['base']['vtbpay']['vtbpay_system']);
    $paymetSettings = $PHPShopOrm->select();

    // Initialize VTBPay logger
    $logger = get_vtbpay_logger($paymetSettings);

    // Check if the payment method matches VTBPay and the order routing is 'END'
    if ($rout == 'END' && $obj->PHPShopPayment->objRow['name'] == $paymetSettings['title']) {
        try {
            // Check if the payment URL is present and not empty in the response.
            if (empty($payUrl = get_pay_url($obj, $value, $paymetSettings, $logger))) {
                throw new \Exception('   URL- .');
            }

            // Set payment information and build payment form
            if (isset($paymetSettings['payment_info']) && !empty($paymentInfo = $paymetSettings['payment_info'])) {
                $obj->set('payment_info', $paymentInfo);
            }

            $obj->set('payment_forma', PHPShopText::button($paymetSettings['btn_text'], "window.location.replace('{$payUrl}')", 'btn btn-primary btn-pill'));
            $form = ParseTemplateReturn($GLOBALS['SysValue']['templates']['vtbpay']['vtbpay_payment_form'], true);

            // Clear the shopping cart
            unset($_SESSION['cart']);

        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            $logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                $e->getMessage(),
                $value['ouid'] ?? ''
            ), $context);

            $obj->set('mesageText', '<span style="color:red;">' . PHPShopString::utf8_win1251($e->getMessage()) . '</span>');
            $form = ParseTemplateReturn($GLOBALS['SysValue']['templates']['order_forma_mesage']);
        }

        // Set order message with payment form
        $obj->set('orderMesage', $form);
    }
}


/**
 *     
 *
 * @return string
 */
function get_pay_url($obj, $value, $paymetSettings, $logger)
{
    $mail = $_POST['mail'] ?? $paymetSettings['email_fiscal'];

    // Check if email is set in the POST data
    if (empty($mail)) {
        throw new \Exception('   .');
    }

    // Check if the total amount is set in the order object
    if (!isset($obj->total) || empty($total = $obj->total)) {
        throw new \Exception('   .');
    }

    // Build the return URL
    $returnUrl = 'http://' . $_SERVER['HTTP_HOST'] . '/phpshop/modules/vtbpay/payment/result.php?type=return';

    //  
    $items = [];
    if (
        !$paymetSettings['two_stage'] &&
        (bool) $paymetSettings['enable_fiscal']
    ) $items = get_items($obj, $paymetSettings);

    // Logging input
    $logger->debug(
        __FUNCTION__ . ' > getOrderLink - INPUT: ', [
        'arguments' => [
            'ouid' => $value['ouid'],
            'email' => $mail,
            'time' => time(),
            'total' => $total,
            'return_url' => $returnUrl,
            'two_stage' => $paymetSettings['two_stage'],
            'items' => $items
        ]
    ]);

    // Get the payment link from VTB API
    $response = get_vtb_api($paymetSettings)->getOrderLink(
        $value['ouid'],
        $mail,
        time(),
        $total,
        $returnUrl,
        (bool) $paymetSettings['two_stage'],
        $items
    );

    // Logging output
    $logger->debug(
        __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
        'response' => $response
    ]);

    return $response['object']['payUrl'] ?? '';
}


/**
 *      .
 *
 * @return array  .
 */
function get_items($obj, $paymetSettings): array
{
    $orders = unserialize($obj->order);
    $items = [];
    $key = 1;
    //      
    $fiscalSettings = [
        'measure_fiscal' => 0,
        'tax_type_fiscal' => 'none',
        'payment_type_fiscal' => 'full_prepayment',
        'payment_subject_fiscal' => 1
    ];

    //    
    foreach ($orders['Cart']['cart'] as $item) {

        $PHPShopProduct = new PHPShopProduct($item['id']);
        $productFiscal = [];
        //      
        foreach ($fiscalSettings as $name => $default) {
            $productFiscal[$name] = $PHPShopProduct->getParam('vtbpay_' . $name) ?: '-';
            if ($productFiscal[$name] === '-') $productFiscal[$name] = $paymetSettings[$name] ?? $default;
        }

        // 
        if ((float) $obj->discount > 0 && empty($item['promo_price']))
            $price = $item['price'] - ($item['price'] * (float) $obj->discount / 100);
        else
            $price = $item['price'];

        // 
        if ($obj->bonus_minus > 0 and $key == 1) {
            $price = $price - $obj->bonus_minus;
        }

        $price = floatval($price);
        $amount = floatval($price * (int) $item['num']);

        $items[] = [
            'positionId' => $key,
            'name' => PHPShopString::win_utf8($item['name']),
            'code' => (string) $item['uid'] ?: $item['id'],
            'price' => $price,
            'measure' => (int) $productFiscal['measure_fiscal'],
            'quantity' => (int) $item['num'],
            'taxParams' => [
                'taxType' => $productFiscal['tax_type_fiscal']
            ],
            'paymentType' => $productFiscal['payment_type_fiscal'],
            'paymentSubject' => (int) $productFiscal['payment_subject_fiscal'],
            'amount' => $amount
        ];

        $key++;
    }

    if ($obj->delivery > 0) {
        $delivery = $obj->PHPShopDelivery->objRow;
        $deliveryPrice = floatval($delivery['price']);
        $items[] = [
            'positionId' => $key,
            'name' => PHPShopString::win_utf8($delivery['city'] ?? ''),
            'price' => $deliveryPrice,
            'measure' => 0,
            'quantity' => 1,
            'taxParams' => [
                'taxType' => $paymetSettings['tax_type_fiscal'] ?? 'none'
            ],
            'paymentType' => $paymetSettings['payment_type_delivery_fiscal'] ?? 'full_prepayment',
            'paymentSubject' => 4,
            'amount' => $deliveryPrice
        ];
    }

    return $items;
}


/**
 *      API .
 *
 * @return VtbApi     API .
 */
function get_vtb_api($paymetSettings): VtbApi
{
    return new VtbApi(
        html_entity_decode($paymetSettings['client_id']),
        html_entity_decode($paymetSettings['client_secret']),
        (bool) $paymetSettings['test_mode'],
        $paymetSettings['merchant_authorization'] ?: ''
    );
}


/**
 *     .
 */
function get_vtbpay_logger($paymetSettings): VtbPayLogger
{
    $logger = VtbPayLogger::getInstance();
    $logging = (bool) $paymetSettings['logging'];

    return $logger->setOption('additionalCommonText', 'payment-' . rand(1111, 9999))
        ->setLogFilePath(dirname(__DIR__) . '/log/vtbpay-' . date('d-m-Y') . '.log')
        ->setCustomRecording(function ($message) use ($logger, $logging) {
            //   
            if ($logging) $logger->writeToFile(PHPShopString::utf8_win1251($message));
        }, VtbPayLogger::LOG_LEVEL_DEBUG)
        ->setCustomRecording(function ($message) use ($logger, $logging) {
            //   
            $logger->writeToFile(PHPShopString::utf8_win1251($message));
        }, VtbPayLogger::LOG_LEVEL_ERROR);
}


// Add the payment hook handler
$addHandler = [
    'send_to_order' => 'send_to_order_mod_vtbpay_hook'
];
